package com.dayou.service.workflow;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.dto.WorkNodeCommit;
import com.dayou.dto.WorkNodeDTO;
import com.dayou.entity.WorkFlow;
import com.dayou.entity.WorkFlowNodeInstance;
import com.dayou.entity.WorkNodeTask;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.enums.workflow.NodeLogEnum;
import com.dayou.enums.workflow.NodeStateEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.WorkFlowMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.mapper.WorkTaskRecordMapper;
import com.dayou.utils.LoginContext;
import com.dayou.vo.NameCodeVO;
import com.dayou.workflow.handler.MajorNodeHandler;
import com.dayou.workflow.config.WorkNodeProcessable;
import com.dayou.workflow.handler.PersonalNodeHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import static com.dayou.enums.MainBusinessEnum.MAJOR_BUSINESS;
import static com.dayou.enums.MainBusinessEnum.PERSONAL_BUSINESS;
import static com.dayou.enums.workflow.NodeLogEnum.*;

/**
 * <p>
 * 工作流 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-10-17
 */
@Service
public class WorkFlowServiceImpl extends ServiceImpl<WorkFlowMapper, WorkFlow> implements IWorkFlowService {

    @Autowired
    private IWorkFlowNodeInstanceService workFlowNodeInstanceService;

    @Autowired
    private MajorNodeHandler majorNodeHandler;

    @Autowired
    private PersonalNodeHandler personalNodeHandler;

    @Autowired
    private IWorkNodeService workNodeService;

    @Autowired
    private IWorkTaskRecordService workTaskRecordService;

    @Autowired
    private WorkTaskRecordMapper workTaskRecordMapper;


    @Override
    @SuppressWarnings("unchecked")
    public Page<WorkFlow> selectPage(Page page,WorkFlow workFlow){
        return this.page(page, new QueryWrapper<WorkFlow>(workFlow));
    }


    @Override
    public WorkFlow detail(Long id){
        return this.getById(id);
    }

    @Override
    public Long add(WorkFlow workFlow){
        this.save(workFlow);
        return workFlow.getId();
    }

    @Override
    public Boolean update(WorkFlow workFlow){
        return  this.updateById(workFlow);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public List<NameCodeVO> flowsEnum() {
        List<MainBusinessEnum> enums = Arrays.asList(MainBusinessEnum.values());
        return enums.stream().map(x -> {
            NameCodeVO nameCodeVO = new NameCodeVO();
            nameCodeVO.setCode(x.getCode());
            nameCodeVO.setName(x.getMsg());
            return nameCodeVO;
        }).collect(Collectors.toList());
    }

    @Override
    public Boolean commit(WorkNodeCommit workNodeCommit) {
        WorkFlowNodeInstance instanceNode = workFlowNodeInstanceService.getById(workNodeCommit.getInstanceNodeId());
        if (instanceNode==null){
            ErrorCode.throwBusinessException(ErrorCode.CURRENT_NODE_NOT_EXISTS);
        }
        //校验节点提交权限
//        Set<Long> handlerIds = workTaskRecordMapper.instanceNodeTasks(workNodeCommit.getInstanceNodeId()).stream().map(WorkNodeInstanceTaskDTO::getHandlerId).collect(Collectors.toSet());
//        if (!handlerIds.contains(LoginContext.getCurrentUserId())){
//            ErrorCode.throwBusinessException(ErrorCode.PERMISSION_ERROR);
//        }
        MainBusinessEnum businessEnum = MainBusinessEnum.valueOf(instanceNode.getBusinessType());

        if (businessEnum.equals(MAJOR_BUSINESS)){
            return handleByState(majorNodeHandler,workNodeCommit);
        }
        if (businessEnum.equals(PERSONAL_BUSINESS)){
            return handleByState(personalNodeHandler,workNodeCommit);
        }
        return Boolean.FALSE;
    }

    /**
     * 业务下单
     * @param businessId
     * @param remark
     */
    @Override
    public WorkNodeCommit openingOrder(MainBusinessEnum mainBusinessEnum,Long businessId,String businessSubId, String remark){
        WorkFlow majorBusinessFlow = this.getOne(new LambdaQueryWrapper<WorkFlow>()
                .eq(WorkFlow::getCode, mainBusinessEnum.getCode())
                .eq(WorkFlow::getState, Boolean.FALSE)
                .eq(BaseEntity::getDeleted, Boolean.FALSE));

        if (majorBusinessFlow == null){
            ErrorCode.throwBusinessException(ErrorCode.WORKFLOW_NOT_EXISTS,mainBusinessEnum.getMsg());
        }

        WorkNodeDTO openingOrderNode = workNodeService.getOpeningOrderNode(majorBusinessFlow.getId());
        if (openingOrderNode == null){
            ErrorCode.throwBusinessException(ErrorCode.OPENING_NODE_NOT_EXISTS,mainBusinessEnum.getMsg());
        }

        WorkFlowNodeInstance instance = new WorkFlowNodeInstance();
        instance.setFlowId(majorBusinessFlow.getId());
        instance.setNodeId(openingOrderNode.getId());
        instance.setBusinessId(businessId);
        instance.setBusinessSubId(businessSubId);
        instance.setBusinessType(mainBusinessEnum.getCode());
        instance.setSequence(0);
        instance.setState(NodeStateEnum.FINISHED.name());
        instance.setComment(remark);
        workFlowNodeInstanceService.save(instance);

        List<WorkNodeTask> tasks = openingOrderNode.getTasks();


        if (CollectionUtil.isNotEmpty(tasks)){
            if (mainBusinessEnum.equals(PERSONAL_BUSINESS)){
                //业务成功下单后，将下单任务标记已完成。
                if (workTaskRecordService.doCreateNodeTaskRecord(majorNodeHandler,tasks, instance, LoginContext.getCurrentUserId())){
                    workTaskRecordMapper.finishOpening(instance.getId(),majorBusinessFlow.getId());
                }
            }else{
                //业务成功下单后，将下单任务标记已完成。
                if (workTaskRecordService.doCreateNodeTaskRecord(personalNodeHandler,tasks, instance, LoginContext.getCurrentUserId())){
                    workTaskRecordMapper.finishOpening(instance.getId(),majorBusinessFlow.getId());
                }
            }

        }

        //构造下单提交对象
        WorkNodeCommit commit = new WorkNodeCommit();
        commit.setInstanceNodeId(instance.getId());
        commit.setBusinessSubId(businessSubId);
        commit.setComments(remark);
        commit.setState(NodeLogEnum.PASS);
        return commit;
    }
    private Boolean handleByState(WorkNodeProcessable workNodeProcessable,WorkNodeCommit workNodeCommit){
        NodeLogEnum state = workNodeCommit.getState();
        if (state == null){
            ErrorCode.throwBusinessException(ErrorCode.COMMIT_STATE_ERROR);
        }
        if (state.equals(PASS)){
            return workNodeProcessable.forward(workNodeCommit);
        }
        if (state.equals(REVERSE)){
            return workNodeProcessable.goBack(workNodeCommit);
        }
        if (state.equals(RESTART)){
            return workNodeProcessable.reset(workNodeCommit);
        }
        if (state.equals(TERMINATE)){
            return workNodeProcessable.ending(workNodeCommit);
        }
        if (state.equals(SKIP)){
            return workNodeProcessable.skip(workNodeCommit);
        }
        return Boolean.FALSE;
    }
}
